@@ -8,13 +8,23 @@ int main()
 {
   std::vector<int> primes;
 
-  // Store the primes <= 1000
+  // Store primes <= 1000
   primesieve::generate_primes(1000, &primes);
 
   primes.clear();
 
-  // Store the first 1000 primes
+  // Store primes inside [1000, 2000]
+  primesieve::generate_primes(1000, 2000, &primes);
+
+  primes.clear();
+
+  // Store first 1000 primes
   primesieve::generate_n_primes(1000, &primes);
 
+  primes.clear();
+
+  // Store first 10 primes >= 1000
+  primesieve::generate_n_primes(10, 1000, &primes);
+
   return 0;
 }
