@@ -11,7 +11,7 @@
 #define ERATSMALL_HPP
 
 #include "Bucket.hpp"
-#include "noinline.hpp"
+#include "macros.hpp"
 #include "Wheel.hpp"
 
 #include <stdint.h>
